@@ -15,7 +15,7 @@ from collections.abc import Sequence
 import thermosteam as tmo
 import numpy as np
 
-__all__ = ('Chemicals', 'CompiledChemicals')
+__all__ = ('Chemicals', 'CompiledChemicals', 'ChemicalDraft', 'ChemicalsOutline')
 setattr = object.__setattr__
     
 # %% Functions
@@ -24,12 +24,138 @@ def must_compile(*args, **kwargs): # pragma: no cover
     raise TypeError("method valid only for compiled chemicals; "
                     "run <Chemicals>.compile() to compile")
 
-def chemical_data_array(chemicals, attr, dtype=float):
+def chemical_data_array(chemicals, attr):
     getfield = getattr
-    data = np.asarray([getfield(i, attr) for i in chemicals], dtype)
+    data = np.array([getfield(i, attr) for i in chemicals], dtype=float)
     data.setflags(0)
     return data
+ 
+def prepare(chemicals, skip_checks):
+    free_energies = ('H', 'S', 'H_excess', 'S_excess')
+    for chemical in chemicals:
+        if chemical.get_missing_properties(free_energies):
+            chemical.reset_free_energies()
+        if skip_checks: continue
+        key_properties = chemical.get_key_property_names()
+        missing_properties = chemical.get_missing_properties(key_properties)
+        if not missing_properties: continue
+        missing = utils.repr_listed_values(missing_properties)
+        raise RuntimeError(
+            f"{chemical} is missing key thermodynamic properties ({missing}); "
+            "use the `<Chemical>.get_missing_properties()` to check "
+            "all missing properties"
+        )
+
+# %% Outline
+
+class ChemicalDraft:
+    """
+    Create a draft of a chemical without actually loading the chemical.
+    This allows for extending and modifying an outline of a property package
+    before loading all the chemicals.
+    """
+    def __init__(self, ID, CAS=None, *args, **kwargs):
+        self.ID = ID
+        self.CAS = CAS
+        self.args = args
+        self.kwargs = kwargs
+        
+    def copy(self, CAS=None, **data):
+        new = ChemicalDraft(self.ID, CAS=self.CAS)
+        for i, j in data.items(): setattr(new, i, j)
+        
+    def to_chemical(self):
+        dct = self.__dict__.copy()
+        chemical = Chemical(
+            dct.pop('ID'), *dct.pop('args'), CAS=dct.pop('CAS'), **dct.pop('kwargs')
+        )
+        for i, j in dct.items(): setattr(chemical, i, j)
+        return chemical
+        
+    def __repr__(self):
+        return f'{type(self).__name__}({self.ID})'
+
+
+class ChemicalsOutline:
+    """
+    Create an outline of chemicals without actually loading the chemicals.
+    This allows for extending and modifying an outline of a property package
+    before loading all the chemicals.
+    """
+    def __new__(cls, chemicals, cache=None):
+        self = super().__new__(cls)
+        isa = isinstance
+        setfield = setattr
+        chemicals = [i if isa(i, ChemicalDraft) else ChemicalDraft(i, cache=cache) for i in chemicals]
+        for i in chemicals: setfield(self, i.ID, i)
+        return self
     
+    def __getnewargs__(self):
+        return (tuple(self),)
+    
+    def __setattr__(self, ID, chemical):
+        raise TypeError("can't set attribute; use <ChemicalsOutline>.append instead")
+    
+    def __setitem__(self, ID, chemical):
+        raise TypeError("can't set item; use <ChemicalsOutline>.append instead")
+    
+    def __getitem__(self, key):
+        dct = self.__dict__
+        try:
+            if isinstance(key, str):
+                return dct[key]
+            else:
+                return [dct[i] for i in key]
+        except KeyError as key_error:
+            raise UndefinedChemicalAlias(key_error.args[0])
+    
+    def copy(self):
+        copy = object.__new__(self.__class__)
+        for chem in self: setattr(copy, chem.ID, chem)
+        return copy
+    
+    def append(self, chemical):
+        if isinstance(chemical, str):
+            chemical = ChemicalDraft(chemical)
+        elif not isinstance(chemical, ChemicalDraft):
+            raise TypeError("only 'ChemicalDraft' objects can be appended, "
+                           f"not '{type(chemical).__name__}'")
+        ID = chemical.ID
+        if ID in self.__dict__:
+            raise ValueError(f"{ID} already defined in chemicals")
+        setattr(self, ID, chemical)
+    
+    def extend(self, chemicals):
+        if isinstance(chemicals, ChemicalsOutline):
+            self.__dict__.update(chemicals.__dict__)
+        else:
+            for chemical in chemicals: self.append(chemical)
+    
+    def to_chemicals(self):
+        return Chemicals([i.to_chemical() for i in self])
+    
+    kwarray = array = index = indices = must_compile
+        
+    def show(self):
+        print(self)
+    _ipython_display_ = show
+    
+    def __len__(self):
+        return len(self.__dict__)
+    
+    def __contains__(self, chemical):
+        if isinstance(chemical, str):
+            return chemical in self.__dict__
+        elif hasattr(chemical, 'ID'):
+            return chemical.ID in self.__dict__
+        else: # pragma: no cover
+            return False
+    
+    def __iter__(self):
+        yield from self.__dict__.values()
+    
+    def __repr__(self):
+        return f"{type(self).__name__}([{', '.join(self.__dict__)}])"
 
 # %% Chemicals
 
@@ -115,12 +241,12 @@ class Chemicals:
     UndefinedChemicalAlias: 'Butane'
     
     """
-    def __new__(cls, chemicals, cache=None):
+    def __new__(cls, chemicals, cache=None, db='default'):
         self = super().__new__(cls)
         isa = isinstance
         setfield = setattr
         CASs = set()
-        chemicals = [i if isa(i, Chemical) else Chemical(i, cache=cache) for i in chemicals]
+        chemicals = [i if isa(i, Chemical) else Chemical(i, cache=cache, db=db) for i in chemicals]
         for i in chemicals:
             CAS = i.CAS
             if CAS in CASs: continue
@@ -228,8 +354,9 @@ class Chemicals:
         
         """
         chemicals = tuple(self)
+        prepare(chemicals, skip_checks)
         setattr(self, '__class__', CompiledChemicals)
-        try: self._compile(chemicals, skip_checks)
+        try: self._compile(chemicals)
         except Exception as error:
             setattr(self, '__class__', Chemicals)
             setattr(self, '__dict__', {i.ID: i for i in chemicals})
@@ -336,10 +463,10 @@ class CompiledChemicals(Chemicals):
     _cache = {}
     
     def __new__(cls, chemicals, cache=None):
-        chemicals = tmo.Chemicals(chemicals)
+        chemicals = Chemicals(chemicals)
         chemicals_tuple = tuple(chemicals) 
         cache = cls._cache
-        if chemicals in cache:
+        if chemicals_tuple in cache:
             self = cache[chemicals]
         else:
             chemicals.compile(cache)
@@ -408,7 +535,7 @@ class CompiledChemicals(Chemicals):
         >>> s2.imol['Alcohol']
         2.0
         >>> s2.imol['l', 'Alcohol']
-        0.678
+        0.6793
         >>> s2.imol['l', ('Water', 'Alcohol')]
         array([1.321, 0.679])
         
@@ -467,13 +594,20 @@ class CompiledChemicals(Chemicals):
             composition = np.ones(len(IDs))
         elif len(composition) != len(IDs): 
             raise ValueError('length of IDs and composition must be the same')
-        for i in IDs: 
-            if i in self._group_wt_compositions:
-                raise ValueError(f"'{i}' is a group; cannot define new group using other groups")
-        index = self.indices(IDs)
+        new_IDs = []
+        new_composition = []
+        group_compositions = self._group_wt_compositions if wt else self._group_mol_compositions
+        for n, i in enumerate(IDs): 
+            if i in group_compositions:
+                new_IDs.extend([j.ID for j in self[i]])
+                new_composition.extend(composition[n] * group_compositions[i])
+            else:
+                new_IDs.append(i)
+                new_composition.append(composition[n])
+        index = self.indices(new_IDs)
         self.__dict__[name] = [self.tuple[i] for i in index]
         self._index[name] = index
-        composition = np.asarray(composition, float)
+        composition = np.asarray(new_composition, float)
         if wt:
             composition_wt = composition
             composition_mol = composition / self.MW[index]
@@ -488,6 +622,14 @@ class CompiledChemicals(Chemicals):
         """All defined chemical groups."""
         return frozenset(self._group_mol_compositions)
     
+    def chemical_group_members(self, name) -> list[str]:
+        index = self._index
+        if name in index:
+            IDs = self.IDs
+            return [IDs[i] for i in index[name]]
+        else:
+            raise ValueError(f"chemical group '{name}' does not exist")
+    
     def refresh_constants(self):
         """
         Refresh constant arrays according to their chemical values,
@@ -520,22 +662,9 @@ class CompiledChemicals(Chemicals):
         reactions = [i.get_combustion_reaction(self) for i in self]
         return tmo.reaction.ParallelReaction([i for i in reactions if i is not None])
 
-    def _compile(self, chemicals, skip_checks=False):
+    def _compile(self, chemicals):
         dct = self.__dict__
         tuple_ = tuple
-        free_energies = ('H', 'S', 'H_excess', 'S_excess')
-        for chemical in chemicals:
-            if chemical.get_missing_properties(free_energies):
-                chemical.reset_free_energies()
-            if skip_checks: continue
-            key_properties = chemical.get_key_property_names()
-            missing_properties = chemical.get_missing_properties(key_properties)
-            if not missing_properties: continue
-            missing = utils.repr_listed_values(missing_properties)
-            raise RuntimeError(
-                f"{chemical} is missing key thermodynamic properties ({missing}); "
-                "use the `<Chemical>.get_missing_properties()` to check "
-                "all missing properties")
         IDs = tuple_([i.ID for i in chemicals])
         CAS = tuple_([i.CAS for i in chemicals])
         size = len(IDs)
@@ -562,7 +691,11 @@ class CompiledChemicals(Chemicals):
             if not i.iupac_name: i.iupac_name = ()
             elif isa(i.iupac_name, str):
                 i.iupac_name = (i.iupac_name,)
-            all_names = set([*i.iupac_name, *i.aliases, i.common_name, i.formula])
+            all_names = list([*i.iupac_name, *i.aliases, i.common_name, i.formula])
+            if i.common_name:
+                all_names.append(i.common_name.capitalize())
+                all_names.append(i.common_name.upper())
+            all_names = set(all_names)
             all_names_list.append(all_names)
             for name in all_names:
                 if not name: continue
@@ -577,6 +710,7 @@ class CompiledChemicals(Chemicals):
                     self.set_alias(ID, name)
         vle_chemicals = []
         lle_chemicals = []
+        vlle_chemicals = []
         heavy_chemicals = []
         light_chemicals = []
         for i in chemicals:
@@ -586,20 +720,23 @@ class CompiledChemicals(Chemicals):
                     heavy_chemicals.append(i)
                     if i.Dortmund or i.UNIFAC or i.NIST or i.PSRK:
                         lle_chemicals.append(i)
-                    if i.N_solutes is None: i._N_solutes = 0
+                        vlle_chemicals.append(i)
                 elif locked_phase == 'g':
                     light_chemicals.append(i)
                 else:
                     raise Exception('chemical locked state has an invalid phase')
             else:
                 vle_chemicals.append(i)
-                if i.Dortmund or i.UNIFAC or i.NIST or i.PSRK: lle_chemicals.append(i)
+                vlle_chemicals.append(i)
+                if i.Dortmund or i.UNIFAC or i.NIST or i.PSRK: 
+                    lle_chemicals.append(i)
         dct['vle_chemicals'] = tuple_(vle_chemicals)
         dct['lle_chemicals'] = tuple_(lle_chemicals)
         dct['heavy_chemicals'] = tuple_(heavy_chemicals)
         dct['light_chemicals'] = tuple_(light_chemicals)
         dct['_vle_index'] = [index[i.ID] for i in vle_chemicals]
         dct['_lle_index'] = [index[i.ID] for i in lle_chemicals]
+        dct['_vlle_index'] = [index[i.ID] for i in vlle_chemicals]
         dct['_heavy_solutes'] = chemical_data_array(heavy_chemicals, 'N_solutes')
         dct['_heavy_indices'] = [index[i.ID] for i in heavy_chemicals]
         dct['_light_indices'] = [index[i.ID] for i in light_chemicals]
@@ -648,9 +785,8 @@ class CompiledChemicals(Chemicals):
         'C4H10O2'
         
         """
-        isvalid = utils.is_valid_ID      
         for i in self.get_aliases(ID):
-            if isvalid(i): return i        
+            if all([s not in i for s in ('-', ',')]): return i        
     
     get_parsable_synonym = get_parsable_alias
     
@@ -672,7 +808,7 @@ class CompiledChemicals(Chemicals):
         >>> aliases = chemicals.get_aliases('Water')
         >>> aliases.sort()
         >>> aliases
-        ['7732-18-5', 'H2O', 'Water', 'oxidane', 'water']
+        ['7732-18-5', 'H2O', 'WATER', 'Water', 'oxidane', 'water']
         
         """
         k = self._index[ID]
@@ -788,7 +924,7 @@ class CompiledChemicals(Chemicals):
         """
         array =  np.zeros(self.size) 
         index, kind = self._get_index_and_kind(tuple(IDs))
-        if kind == 0:
+        if kind == 0 or kind == 3:
             array[index] = data
         elif kind == 1:
             raise ValueError('cannot create array by chemical groups')
@@ -829,8 +965,9 @@ class CompiledChemicals(Chemicals):
         1.0
         
         """
-        array = self.array(IDs, data)
-        return ChemicalIndexer.from_data(array, chemicals=self)
+        indexer = ChemicalIndexer.blank(None, self)
+        indexer[IDs] = data
+        return indexer
 
     def ikwarray(self, ID_data):
         """
@@ -862,9 +999,61 @@ class CompiledChemicals(Chemicals):
         1.0
         
         """
-        array = self.kwarray(ID_data)
-        return ChemicalIndexer.from_data(array, chemicals=self)
-
+        indexer = ChemicalIndexer.blank(None, self)
+        IDs, data = zip(*ID_data.items())
+        indexer[IDs] = data
+        return indexer
+    
+    def split(self, IDs, data):
+        """
+        Return an array with entries that correspond to the ordered chemical IDs.
+        
+        Parameters
+        ----------
+        IDs : iterable
+            Compound IDs.
+        data : array_like
+            Data corresponding to IDs.
+            
+        Examples
+        --------
+        >>> from thermosteam import CompiledChemicals
+        >>> chemicals = CompiledChemicals(['Water', 'Ethanol'], cache=True)
+        >>> chemicals.split(['Water'], [1])
+        array([1., 0.])
+        
+        """
+        array =  np.zeros(self.size) 
+        index, kind = self._get_index_and_kind(tuple(IDs))
+        if kind == 0 or kind == 3:
+            array[index] = data
+        elif kind == 1:
+            array[index] = data
+        elif kind == 2:
+            for i, j in zip(index, data): array[i] = j
+        else:
+            raise IndexError('unknown error')
+        return array
+    
+    def kwsplit(self, ID_data):
+        """
+        Return an array with entries that correspond to the ordered chemical IDs.
+        
+        Parameters
+        ----------
+        ID_data : dict[str, float]
+            ID-data pairs.
+            
+        Examples
+        --------
+        >>> from thermosteam import CompiledChemicals
+        >>> chemicals = CompiledChemicals(['Water', 'Ethanol'], cache=True)
+        >>> chemicals.kwsplit(dict(Water=1))
+        array([1., 0.])
+        
+        """
+        return self.split(*zip(*ID_data.items()))
+    
     def isplit(self, split, order=None):
         """
         Create a SplitIndexer object that represents chemical splits.
@@ -919,17 +1108,14 @@ class CompiledChemicals(Chemicals):
         if isinstance(split, dict):
             assert not order, "cannot pass 'order' key word argument when split is a dictionary"
             order, split = zip(*split.items())
-        
         if order:
             isplit = SplitIndexer.blank(chemicals=self)
-            isplit[tuple(order)] = split
+            isplit[order] = split
         elif hasattr(split, '__len__'):
-            isplit = SplitIndexer.from_data(np.asarray(split),
-                                            chemicals=self)
+            isplit = SplitIndexer.from_data(split, chemicals=self)
         else:
-            split = split * np.ones(self.size)
-            isplit = SplitIndexer.from_data(split,
-                                            chemicals=self)
+            isplit = SplitIndexer.blank(chemicals=self)
+            isplit[...] = split
         return isplit
 
     def index(self, ID):
@@ -1073,18 +1259,25 @@ class CompiledChemicals(Chemicals):
             return index_cache[key]
         except KeyError:
             isa = isinstance
-            kind = 0 # [int] Kind of index: 0 - normal, 1 - chemical group, 2 - nested chemical group
+            # [int|None] Kind of index:
+            # None - all
+            # 0 - chemical
+            # 1 - chemical group
+            # 2 - nested chemical group
+            # 3 - array
             if isa(key, str):
                 index = self.index(key)
-                if isa(index, list): kind = 1 
+                kind = 0 if isa(index, int) else 1
             elif isa(key, tuple):
                 index = self.indices(key)
                 for i in index:
                     if isa(i, list): 
                         kind = 2
                         break
+                else:
+                    kind = 3
             elif key is ...:
-                index = slice(None)
+                kind = index = None
             else: # pragma: no cover
                 raise TypeError("only strings, sequences of strings, and ellipsis are valid index keys")
             index_cache[key] = index, kind
@@ -1139,5 +1332,21 @@ class CompiledChemicals(Chemicals):
         """
         return [i for i in self._lle_index if i in nonzeros]
     
+    def get_vlle_indices(self, nonzeros):
+        """
+        Return indices of species in liquid-liquid equilibrium given an array
+        dictating whether or not the chemicals are present.
+        
+        Examples
+        --------
+        >>> from thermosteam import CompiledChemicals
+        >>> chemicals = CompiledChemicals(['Water', 'Methanol', 'Ethanol'])
+        >>> data = chemicals.kwarray(dict(Water=2., Ethanol=1.))
+        >>> chemicals.get_vlle_indices(data[data!=0])
+        [1, 2]
+        
+        """
+        return [i for i in self._vlle_index if i in nonzeros]
+    
     def __repr__(self):
         return f"{type(self).__name__}([{', '.join(self.IDs)}])"
