@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
-# Copyright (C) 2020-2021, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# Copyright (C) 2020-2023, Yoel Cortes-Pena <yoelcortes@gmail.com>
 # 
 # This module is under the UIUC open-source license. See 
 # github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
@@ -67,4 +67,10 @@ class ThermalCondition:
         return iter((self._T, self._P))
         
     def __repr__(self):
-        return f"{type(self).__name__}(T={self.T:.2f}, P={self.P:.6g})"
\ No newline at end of file
+        return f"{type(self).__name__}(T={self.T:.2f}, P={self.P:.6g})"
+    
+class MockThermalCondition:
+    __slots__ = ()
+    def in_equilibrium(self, other): return False
+
+mock_thermal_condition = MockThermalCondition()    
\ No newline at end of file
