@@ -10,9 +10,15 @@
 
 from . import functor
 from . import phase_handle
+from . import sparse
+from . import dictionary_view
 
 __all__ = (*functor.__all__,
-           *phase_handle.__all__)
+           *phase_handle.__all__,
+           *sparse.__all__,
+           *dictionary_view.__all__)
 
 from .functor import *
 from .phase_handle import *
+from .sparse import *
+from .dictionary_view import *
