@@ -15,22 +15,11 @@
 # https://github.com/CalebBell/thermo/blob/master/LICENSE.txt for details.
 """
 """
-
 from . import (
-    dippr,
     elements,
-    heat_capacity,
     identifiers,
-    interface,
-    permittivity,
-    phase_change,
     reaction,
-    thermal_conductivity,
-    vapor_pressure,
-    viscosity,
-    volume,
 )
-
 import chemicals
 from chemicals import *
 
