@@ -17,10 +17,11 @@ __all__ = []
 from . import ideal
 from . import domain
 from . import activity_coefficients
+from . import poyinting_correction_factors
 from . import fugacity_coefficients
+from . import partition_coefficients
 from . import dew_point
 from . import bubble_point
-from . import poyinting_correction_factors
 from . import binary_phase_fraction
 from . import fugacities
 from . import vle
@@ -34,33 +35,35 @@ from . import tangential_plane_stability
 __all__ = (*ideal.__all__,
            *domain.__all__,
            *activity_coefficients.__all__,
+           *poyinting_correction_factors.__all__,
+           *fugacity_coefficients.__all__,
+           *partition_coefficients.__all__,
+           *binary_phase_fraction.__all__,
+           *fugacities.__all__,
+           *dew_point.__all__,
+           *bubble_point.__all__,
            *tangential_plane_stability.__all__,
            *vle.__all__,
            *lle.__all__,
            *sle.__all__,
-           *dew_point.__all__,
-           *bubble_point.__all__,
-           *fugacity_coefficients.__all__,
-           *poyinting_correction_factors.__all__,
-           *binary_phase_fraction.__all__,
-           *fugacities.__all__,
            *plot_equilibrium.__all__,
            *flash_package.__all__,
            *vlle.__all__)
 
 from .ideal import *
 from .domain import *
+from .binary_phase_fraction import *
+from .activity_coefficients import *
+from .poyinting_correction_factors import *
+from .fugacity_coefficients import *
+from .fugacities import *
+from .partition_coefficients import *
+from .dew_point import *
+from .bubble_point import *
 from .tangential_plane_stability import *
 from .vle import *
 from .lle import *
 from .sle import *
-from .binary_phase_fraction import *
-from .activity_coefficients import *
-from .fugacity_coefficients import *
-from .poyinting_correction_factors import *
-from .dew_point import *
-from .bubble_point import *
-from .fugacities import *
 from .plot_equilibrium import *
 from .flash_package import *
 from .vlle import *
