@@ -28,7 +28,7 @@ def y_iter(y, y_phi, phi, T, P):
 
 def solve_y(y_phi, phi, T, P, y_guess):
     if isinstance(phi, IdealFugacityCoefficients): return y_phi
-    return flx.wegstein(y_iter, y_phi, 1e-9, args=(y_phi, phi, T, P), 
+    return flx.wegstein(y_iter, y_phi, 1e-12, args=(y_phi, phi, T, P), 
                         checkiter=False,
                         checkconvergence=False, 
                         convergenceiter=5,
@@ -112,12 +112,15 @@ class BubblePoint:
     __slots__ = ('chemicals', 'IDs', 'gamma', 'phi', 'pcf',
                  'Psats', 'Tmin', 'Tmax', 'Pmin', 'Pmax')
     _cached = {}
-    maxiter = 50
-    T_tol = 1e-9
-    P_tol = 1e-3
-    def __new__(cls, chemicals=(), thermo=None):
+    maxiter = 100
+    T_tol = 1e-12
+    P_tol = 1e-6
+    def __new__(cls, chemicals=None, thermo=None):
         thermo = settings.get_default_thermo(thermo)
-        chemicals = tuple(chemicals)
+        if chemicals is None: 
+            chemicals = thermo.chemicals.tuple
+        else:
+            chemicals = tuple(chemicals)
         key = (chemicals, thermo.Gamma, thermo.Phi, thermo.PCF)
         cached = cls._cached
         if key in cached:
