@@ -10,7 +10,8 @@
 import thermosteam as tmo
 import numpy as np
 
-__all__ = ('LiquidFugacities', 'GasFugacities')
+__all__ = ('LiquidFugacities', 'GasFugacities', 'Fugacities',
+           'fugacities_by_phase')
 
 class LiquidFugacities:
     """
@@ -50,18 +51,21 @@ class LiquidFugacities:
         self.gamma = thermo.Gamma(chemicals)
         self.pcf = thermo.PCF(chemicals)
     
-    def coefficient(self, x, T, P=101325.):
+    def unweighted(self, x, T, P=101325.):
         Psats = np.array([i.Psat(T) for i in self.chemicals], dtype=float)
         return self.gamma(x, T) * self.pcf(T, P, Psats) * Psats
     
-    def __call__(self, x, T, P=101325.):
+    def __call__(self, x, T, P=101325., reduce=False):
+        f_reduced = x * self.gamma(x, T)
+        if reduce: return f_reduced
         Psats = np.array([i.Psat(T) for i in self.chemicals], dtype=float)
-        return x * self.gamma(x, T) * self.pcf(T, P, Psats) * Psats
+        return f_reduced * Psats * self.pcf(T, P, Psats)
     
     def __repr__(self):
         chemicals = ", ".join([i.ID for i in self.chemicals])
         return f"{type(self).__name__}([{chemicals}])"
-    
+
+
 class GasFugacities:
     """
     Create a GasFugacities capable of computing fugacities of chemicals
@@ -99,12 +103,41 @@ class GasFugacities:
         self.chemicals = chemicals = tuple(chemicals)
         self.phi = thermo.Phi(chemicals)
     
-    def coefficient(self, y, T, P):
+    def unweighted(self, y, T, P):
         return P * self.phi(y, T, P)
     
-    def __call__(self, y, T, P):
-        return P * self.phi(y, T, P) * y
+    def __call__(self, y, T, P, reduce=False):
+        f_reduced = self.phi(y, T, P) * y
+        if reduce: return f_reduced
+        return P * f_reduced
     
     def __repr__(self):
         chemicals = ", ".join([i.ID for i in self.chemicals])
         return f"{type(self).__name__}([{chemicals}])"
+
+
+class Fugacities:
+    __slots__ = ('fugacities_by_phase',)
+    
+    def __init__(self, chemicals, thermo=None):
+        self.fugacities_by_phase = fugacities_by_phase(chemicals, thermo)
+        
+    def unweighted(self, phase, z, T, P):
+        return self.fugacities_by_phase[phase].unweighted(z, T, P)
+        
+    def __call__(self, phase, z, T, P, reduce=False):
+        return self.fugacities_by_phase[phase](z, T, P, reduce)
+    
+    def __repr__(self):
+        chemicals = ", ".join([i.ID for i in self.chemicals])
+        return f"{type(self).__name__}([{chemicals}])"
+        
+def fugacities_by_phase(chemicals, thermo):
+    thermo = tmo.settings.get_default_thermo(thermo)
+    G = GasFugacities(chemicals, thermo)
+    L = LiquidFugacities(chemicals, thermo)
+    return {
+        'g': G,
+        'l': L,
+        'L': L,
+    }
\ No newline at end of file
