@@ -670,7 +670,7 @@ class ChemicalIndexer(Indexer):
             assert data.ndim == 1, 'material data must be a 1d numpy array'
             assert data.size == self._chemicals.size, ('size of material data must be equal to '
                                                        'size of chemicals')
-            check_phase(phase)
+            if phase is not None: check_phase(phase)
         self._data_cache = {}
         self._parent = parent
         self._lock_phase = lock_phase
@@ -683,18 +683,15 @@ class ChemicalIndexer(Indexer):
     def phase(self, phase):
         parent = self._parent
         if phase == self._phase: return
+        if self._lock_phase: raise AttributeError('phase is locked')
         check_phase(phase)
-        if parent is None:
-            self._phase = phase
-        elif self._lock_phase: 
-            raise AttributeError('phase is locked')
-        else:
+        if parent:
             old_data = self.data
             self.data = parent.data.rows[parent._phase_indexer(phase)]
             dct = self.data.dct
             dct.clear()
             dct.update(old_data.dct)
-            self._phase = phase
+        self._phase = phase
     
     def get_phase_and_composition(self):
         """Return phase and composition."""
