@@ -26,12 +26,11 @@ __all__ = (
     'lle_partition_coefficients',
     'partition', 'lle', 'vle', 
     'material_balance',
-    'chemical_splits',
     'phase_fraction',
 )
 
 def check_partition_infeasibility(infeasible_index, strict, stacklevel=1):
-    if infeasible_index.any():
+    if [i.any() for i in infeasible_index]:
         if strict:
             raise InfeasibleRegion('negative flow rates in equilibrium '
                                    'solution; partition data')
@@ -46,10 +45,10 @@ def check_partition_infeasibility(infeasible_index, strict, stacklevel=1):
 def handle_infeasible_flow_rates(mol, maxmol, strict, stacklevel=1):
     mol = mol
     maxmol = maxmol
-    infeasible_index, = np.where(mol < 0.)
+    infeasible_index = np.where(mol < 0.)
     check_partition_infeasibility(infeasible_index, strict, stacklevel+1)
     mol[infeasible_index] = 0.
-    infeasible_index, = np.where(mol > maxmol)
+    infeasible_index = np.where(mol > maxmol)
     check_partition_infeasibility(infeasible_index, strict, stacklevel+1)
     mol[infeasible_index] = maxmol[infeasible_index]
 
@@ -295,31 +294,6 @@ def partition_coefficients(IDs, top, bottom):
     denominator[denominator < 1e-24] = 1e-24
     return numerator / denominator
 
-def chemical_splits(a, b=None, mixed=None):
-    """
-    Return a ChemicalIndexer with splits for all chemicals to stream `a`.
-    
-    Examples
-    --------
-    >>> import thermosteam as tmo
-    >>> tmo.settings.set_thermo(['Water', 'Ethanol'], cache=True)
-    >>> stream = tmo.Stream('stream', Water=10, Ethanol=10)
-    >>> stream.vle(V=0.5, P=101325)
-    >>> isplits = tmo.separations.chemical_splits(stream['g'], stream['l'])
-    >>> isplits.show()
-    ChemicalIndexer:
-     Water    0.387
-     Ethanol  0.613
-    >>> isplits = tmo.separations.chemical_splits(stream['g'], mixed=stream)
-    >>> isplits.show()
-    ChemicalIndexer:
-     Water    0.387
-     Ethanol  0.613
-     
-    """
-    mixed_mol = mixed.mol.copy() if mixed else a.mol + b.mol
-    return tmo.indexer.ChemicalIndexer.from_data(a.mol / mixed_mol)
-
 def vle_partition_coefficients(top, bottom):
     """
     Return VLE partition coefficients given vapor and liquid streams in equilibrium.
