@@ -14,6 +14,7 @@ from . import registered
 from . import forward
 from . import define_units_of_measure
 from . import chemical_cache
+from . import jitdata
 
 __all__ = (*chemicals_user.__all__,
            *thermo_user.__all__,
@@ -21,7 +22,8 @@ __all__ = (*chemicals_user.__all__,
            *registered.__all__,
            *forward.__all__,
            *define_units_of_measure.__all__,
-           *chemical_cache.__all__)
+           *chemical_cache.__all__,
+           *jitdata.__all__)
 
 from .registered import *
 from .chemicals_user import *
@@ -30,3 +32,4 @@ from .read_only import *
 from .forward import *
 from .define_units_of_measure import *
 from .chemical_cache import *
+from .jitdata import *
